////////////////////////////////////////////////////////////////////////////////
// MENUS97_V2                                                                 //
////////////////////////////////////////////////////////////////////////////////
// Menu Style 97 pour D3 (D2 ?)                                               //
////////////////////////////////////////////////////////////////////////////////
// Version 2.00 Beta                                                          //
// Date de cration           : 02/07/1997                                    //
// Date dernire modification : 15/11/1997                                    //
////////////////////////////////////////////////////////////////////////////////
// Stphane Chova        stephane.chova@hol.fr                                //
// Jean-Luc Mattei       jlucm@club-internet.fr / jlucm@mygale.org            //
////////////////////////////////////////////////////////////////////////////////
// IMPORTANT NOTICE :                                                         //
//                                                                            //
//                                                                            //
// This program is FreeWare                                                   //
//                                                                            //
// Please do not release modified versions of this source code.               //
// If you've made any changes that you think should have been there,          //
// feel free to submit them to me at jlucm@club-internet.fr                   //
////////////////////////////////////////////////////////////////////////////////
// NOTES :                                                                    //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
// REVISIONS :                                                                //
//                                                                            //
//  - 1.20 : * CalcTextWidht Modified ( Accelerators (&) )                    //
//           * HOW TO USE Section                                             //
//           * OnDrawItem Event is called now                                 //
//           * ImageList can be changed at runtime without a GPF              //
//           * Menu can be changed at runtime without GPF                     //
//             (thanks to Edwin Hagen)                                        //
//           * Accelerator Keys works fine now for all items                  //
//           * OnDrawItemEvent is called now                                  //
//  - 1.30 : * CM_MENUCHANGED event added                                     //
//           * Added First Menu Banner                                        //
//  - 1.31 : * Shortcut drawing Bug corrected (thanks to Ian Delisle)         //
//  - 1.32 : * Shortcut drawing Bug (really) corrected (now)                  //
//           * Correct HighLighted Text Color for MenuItems (with Bitmap)     //
//           * Shortcuts disableds for disabled items (thanks - again -       //
//             to Ian Delisle)                                                //
//  - 2.00 : * Added Property Editor (no tag property needed)                 //
//           * Many things changed                                            //
//           * TMenu97 is called TMenu97_V2                                   //
//           * TPopupMenus97_V2 class added                                   //
//           * bakground Bitmap for each item                                 //
//           * Alignment for each item                                        //
////////////////////////////////////////////////////////////////////////////////
// HOW TO USE : V 0.xx to 1.30                                                //
//                                                                            //
//   * Use a standard MainMenu component : M                                  //
//   * Add a Menu97 component            : M97                                //
//   * Link M to Menu property of M97                                         //
//   * Add an image list L and Link L to Image Property of M97                //
//   * Apparence of items is controlled by Tag Property                       //
//       - Standard MenuBarItem  : Tag = -1 (3D look)                         //
//       - Standard PopupItem    : Tag = -2 (Standard Look)                   //
//       - PopupItem with Bitmap : Tag = n of Bitmap in ImageList            //
////////////////////////////////////////////////////////////////////////////////

Unit Menus97_V2;

interface

uses
  Windows, SysUtils, Classes, Messages, Menus, Forms,
  Controls, StdCtrls, Graphics, CommCtrl, ExtCtrls;

{$R Menus97_V2.DCR} // Charge l'icne du composant.
           // Component icon
{$R *.RES} // Charge la marque Check pour l'option Checked.
           // Check Mark

(* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)
type
    TCustomMenus97 = class;

    TMenu97ItemExValue = (mvBrush, mvFont, mvHiliteFont, mvBitmap);
    TMenu97ItemExValues = set of TMenu97ItemExValue;

    TMenu97ItemEx = class(TCollectionItem)
    private
      FFont: TFont;
      FHiliteFont: TFont;
      FBrush: TBrush;
      FAlignment: TAlignment;
      FAssignedValues: TMenu97ItemExValues;
      FImageIndex: Integer;
      FMenuName: string;
      FMenuItem : TMenuItem;
      FBitmap : TBitmap;
      function  DefaultFont: TFont;
      function  DefaultHiliteFont: TFont;
      function  DefaultBrush: TBrush;
      procedure SetBrush(Value: TBrush);
      procedure SetAlignment(Value: TAlignment);
      procedure SetBitmap(Value: TBitmap);
      procedure SetMenuItem(AMenuItem : TMenuItem);
      function  GetMenu97: TCustomMenus97;
      procedure SetFont(Value: TFont);
      procedure SetHiliteFont(Value: TFont);
      function  GetFont: TFont;
      function  GetHiliteFont: TFont;
      function  IsBitmapStored: Boolean;
      function  IsFontStored: Boolean;
      function  IsHiliteFontStored: Boolean;
      function  IsBrushStored: Boolean;
      procedure RestoreDefaults;
      procedure RefreshDefaultFont;
      procedure FontChanged(Sender: TObject);
      procedure RefreshDefaultHiliteFont;
      procedure HiliteFontChanged(Sender: TObject);
    protected
      function GetDisplayName: string; override;
    public
      constructor Create(Collection: TCollection); override;
      destructor Destroy; override;
      procedure Assign(Source: TPersistent); override;
      property AssignedValues: TMenu97ItemExValues read FAssignedValues;
      property MenuName: string read FMenuName write FMenuName;
      property Menu97: TCustomMenus97 read GetMenu97;
    published
      property MenuItem: TMenuItem read FMenuItem write SetMenuItem;
      property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
      property Bitmap: TBitmap read FBitmap write SetBitmap stored IsBitmapStored;
      property Font: TFont read GetFont write SetFont stored IsFontStored;
      property HiliteFont: TFont read GetHiliteFont write SetHiliteFont stored IsHiliteFontStored;
      property Brush: TBrush read FBrush write SetBrush stored IsBrushStored;
      property ImageIndex: Integer read FImageIndex write FImageIndex;
    end;

    TMenu97ItemsEx = class(TCollection)
    private
      FMenu97 : TCustomMenus97;
      function  GetBitmaps (index : TMenuItem) : TBitmap;
      function  GetHiliteFont (index : TMenuItem) : TFont;
      function  GetFont (index : TMenuItem) : TFont;
      function  GetBrush (index : TMenuItem) : TBrush;
      function  GetAlignment (index : TMenuItem) : TAlignment;
      function  GetItem(Index: Integer): TMenu97ItemEx;
      procedure SetItem(Index: Integer; Value: TMenu97ItemEx);
      function  GetItemEx(Index: TMenuItem): TMenu97ItemEx;
      function  GetImageIndex(Index: TMenuItem): Integer;
    protected
      function GetOwner: TPersistent; override;
    public
      constructor Create(AOwner: TCustomMenus97);
      function Add: TMenu97ItemEx;
      property Menu97: TCustomMenus97 read FMenu97;
      property ImageIndex[index : TMenuItem]: Integer read GetImageIndex;
      property Bitmaps[index : TMenuItem]: TBitmap read GetBitmaps;
      property HiliteFont[index : TMenuItem]: TFont read GetHiliteFont;
      property Font[index : TMenuItem]: TFont read GetFont;
      property Brush[index : TMenuItem]: TBrush read GetBrush;
      property Alignment[index : TMenuItem]: TAlignment read GetAlignment;
      property ItemByItem[index : TMenuItem]: TMenu97ItemEx read GetItemEx;
    published
      property Items[Index: Integer]: TMenu97ItemEx read GetItem write SetItem; default;
    end;

  TDrawMenuItemEvent = function(Control: TMenu; Item: TMenuItem; Rect: TRect; State: TOwnerDrawState): Boolean of object;
  TMeasureMenuItemEvent = procedure(Control: TMenu; Item: TMenuItem; var Height, Width: Integer) of object;
  TBannerJustification = (bjCenter, bjTop, bjBottom);

  TCustomMenus97 = class(TComponent)
  private
    FImages: TImageList;
    FImageChangeLink: TChangeLink;
    FCanvas: TCanvas;
    FMenu: TMenu;
    FFont: TFont;
    FBrush: TBrush;
    FItemHeight: Integer;
    FItemWidth: Integer;
    FNewTFormWndProcInstance: Pointer;
    FOldTFormWndProc: Pointer;
    FNewTFormMDIClientWndProcInstance: Pointer;
    FOldTFormMDIClientWndProc: Pointer;
    FOnDrawItem: TDrawMenuItemEvent;
    FOnMeasureItem: TMeasureMenuItemEvent;
    FHiliteBar : TColor;
    FHiliteFont : TFont;
    FDefaultMargin: Integer;
    FDefaultHeight: Integer;
    FTempMenuItem: TMenuItem; // Used for MDI Windows items
    FRadioBitmap : TBitmap;
    FCheckBitmap : TBitmap;
    FBanner: Boolean;
    FBannerText: String;
    FBannerFont: TFont;
    FBannerWidth: Integer;
    FBannerColor: TColor;
    FBannerJustification: TBannerJustification;
    FBannerImage: Integer;
    FImagelist : TimageList;
    FItemsEx: TMenu97ItemsEx;
    Procedure SetImages (Value: TImageList);
    procedure SetInternalMenu(Value: TMenu); virtual;
    procedure SetFont(Value: TFont);
    procedure SetHiliteFont (Value : Tfont);
    procedure SetBannerFont(Value: TFont);
    procedure SetBrush(Value: TBrush);
    procedure WMDrawItem(var Message: TWMDrawItem); message WM_DRAWITEM;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure WMMeasureItem(var Message: TWMMeasureItem); message WM_MEASUREITEM;
    procedure CNMeasureItem(var Message: TWMMeasureItem); message CN_MEASUREITEM;
    procedure DrawItem(Item: TMenuItem; Rect: TRect; State: TOwnerDrawState); virtual;
    procedure MeasureItem(Item: TMenuItem; var Height, Width: Integer); virtual;
    procedure ModifyMenuTree(MenuItems : TMenuItem; Restore: Boolean);
    procedure DrawDisabledImage(Var Rect : TRect; IndexImage : Integer);
    procedure DrawGrayGlyph(Canvas: TCanvas; bmp: TBitmap ; x, y: Integer);
    procedure DrawGlyph(Canvas : TCanvas; bmp:TBitmap; x,y:Integer);
    function  CalcMenuWidth(Item: TMenuItem): Integer;
    procedure CalcHeightItem;
    function  ProcessAccel(Message: TWMMenuChar ): Integer;
    function  FindItemByCommand(Command: Integer): TMenuItem;
    procedure SetItemsEx(Value: TMenu97ItemsEx);
  protected
    procedure Loaded; override;
    procedure Notification (AComponent: TComponent; Operation: TOperation); override;
    procedure FixupReferences;
    property InternalMenu: TMenu read FMenu write SetInternalMenu;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   NewTFormWndProc(Var Message: TMessage);
    procedure   NewTFormMDIClientWndProc(Var Message: TMessage);
    property BannerImage: Integer read FBannerImage write FBannerImage;
    property BannerJustification: TBannerJustification read FBannerJustification write FBannerJustification;
    property BannerWidth: Integer read FBannerWidth write FBannerWidth;
    property BannerText: String read FBannerText write FBannerText;
    property BannerFont: TFont read FBannerFont write SetBannerFont;
    property BannerColor: TColor read FBannerColor write FBannerColor;
    property Banner: Boolean read FBanner write FBanner;
    property OnDrawItem: TDrawMenuItemEvent read FOnDrawItem write FOnDrawItem;
    property OnMeasureItem: TMeasureMenuItemEvent read FOnMeasureItem write FOnMeasureItem;
    property ItemHeight: Integer read FItemHeight write FItemHeight;
    property ItemWidth: Integer read FItemWidth write FItemWidth;
    property Images: TImageList read FImages write SetImages;
    property Font: TFont read FFont write SetFont;
    property Brush: TBrush read FBrush write SetBrush;
    property Canvas: TCanvas read FCanvas;
    property HiliteBar : Tcolor read FHiliteBar write FHiliteBar;
    property HiliteFont : TFont read FHiliteFont write SetHiliteFont;
    property ItemsEx: TMenu97ItemsEx read FItemsEx write SetItemsEx;
  end;

  TCustomMainMenus97 = class(TCustomMenus97)
  private
    procedure SetMainMenu(Value: TMainMenu);
    function  GetMainMenu: TMainMenu;
  public
    property MainMenu: TMainMenu read GetMainMenu write SetMainMenu;
  end;

  TMainMenus97_V2 = class(TCustomMainMenus97)
  published
    property BannerImage;
    property BannerJustification;
    property BannerWidth;
    property BannerText;
    property BannerFont;
    property BannerColor;
    property Banner;
    property MainMenu;
    property OnDrawItem;
    property OnMeasureItem;
    property ItemHeight;
    property ItemWidth;
    Property Images;
    property Font;
    property Brush;
    property Canvas;
    property HiliteBar;
    property HiliteFont;
    property ItemsEx;
  end;

  TCustomPopupMenus97 = class(TCustomMenus97)
  private
    FTPUHandle: THandle;
    FNewTPUtilWndProcInstance: Pointer;
    FOldTPUtilWndProc: Pointer;
    function  GetPopupMenu: TPopupMenu;
    procedure SetPopupMenu(Value: TPopupMenu);
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure NewTPUtilWndProc(Var Message: TMessage);
    property PopupMenu: TPopupMenu read GetPopupMenu write SetPopupMenu;
  end;

  TPopupMenus97_V2 = class(TCustomPopupMenus97)
  published
    property BannerImage;
    property BannerJustification;
    property BannerWidth;
    property BannerText;
    property BannerFont;
    property BannerColor;
    property Banner;
    property PopupMenu;
    property OnDrawItem;
    property OnMeasureItem;
    property ItemHeight;
    property ItemWidth;
    Property Images;
    property Font;
    property Brush;
    property Canvas;
    property HiliteBar;
    property HiliteFont;
    property ItemsEx;
  end;

procedure Register;

Implementation

Const FTPUCount: Integer = 0; // For PopupMenus

Var Pop: TPopupMenu;
    PopItem: TMenuItem;

{ TMenuItemEx }

constructor TMenu97ItemEx.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FAssignedValues:= [];
  FImageIndex:= -1;
  FMenuItem:= nil;
  FBrush:= TBrush.Create;
  FBrush.Assign(DefaultBrush);
  FFont := TFont.Create;
  FFont.Assign(DefaultFont);
  FFont.OnChange := FontChanged;
  FHiliteFont := TFont.Create;
  FHiliteFont.Assign(DefaultHiliteFont);
  FHiliteFont.OnChange := HiliteFontChanged;
  FBitmap:= TBitmap.Create();
  FAlignment := taLeftJustify;
end;

destructor TMenu97ItemEx.Destroy;
begin
  FFont.OnChange:= nil;
  FFont.Free;
  FHiliteFont.OnChange:= nil;
  FHiliteFont.Free;
  FBrush.Free;
  FBitmap.Free;
  MenuItem:= nil;
  inherited Destroy;
end;

procedure TMenu97ItemEx.Assign(Source: TPersistent);
var
  MenuItemEx: TMenu97ItemEx;
begin
  if Source is TMenu97ItemEx then begin
    RestoreDefaults;
    MenuItemEx:= TMenu97ItemEx(Source);
    MenuItem:= MenuItemEx.MenuItem;
    Alignment:= MenuItemEx.Alignment;
    if mvBitmap in MenuItemEx.AssignedValues then
      Bitmap:= MenuItemEx.Bitmap;
    if mvFont in MenuItemEx.AssignedValues then
      Font:= MenuItemEx.Font;
    if mvHiliteFont in MenuItemEx.AssignedValues then
      HiliteFont:= MenuItemEx.HiliteFont;
    if mvBrush in MenuItemEx.AssignedValues then
      Brush:= MenuItemEx.Brush;
  end
  else inherited Assign(Source);
end;

function TMenu97ItemEx.DefaultFont: TFont;
var
  M97: TCustomMenus97;
begin
  M97:= Menu97;
  if Assigned(M97) then
    Result:= M97.Font
  else
    Result:= FFont;
end;

function TMenu97ItemEx.DefaultHiliteFont: TFont;
var
  M97: TCustomMenus97;
begin
  M97:= Menu97;
  if Assigned(M97) then
    Result:= M97.HiliteFont
  else
    Result:= FHiliteFont;
end;

function TMenu97ItemEx.DefaultBrush: TBrush;
var
  M97: TCustomMenus97;
begin
  M97:= Menu97;
  if Assigned(M97) then
    Result:= M97.Brush
  else
    Result:= FBrush;
end;

procedure TMenu97ItemEx.SetBrush(Value: TBrush);
begin
  Include(FAssignedValues, mvBrush);
  FBrush.Assign(Value);
end;

procedure TMenu97ItemEx.RefreshDefaultFont;
var
  Save: TNotifyEvent;
begin
  if mvFont in FAssignedValues then Exit;
  Save:= FFont.OnChange;
  FFont.OnChange := nil;
  try
    FFont.Assign(DefaultFont);
  finally
    FFont.OnChange := Save;
  end;
end;

procedure TMenu97ItemEx.RefreshDefaultHiLiteFont;
var
  Save: TNotifyEvent;
begin
  if mvHiliteFont in FAssignedValues then Exit;
  Save:= FHiliteFont.OnChange;
  FHiliteFont.OnChange:= nil;
  try
    FHiliteFont.Assign(DefaultHiliteFont);
  finally
    FHiliteFont.OnChange:= Save;
  end;
end;

procedure TMenu97ItemEx.RestoreDefaults;
var
  FontAssigned: Boolean;
begin
  FontAssigned:= (mvFont in FAssignedValues) or (mvHiliteFont in FAssignedValues) or
                 (mvBrush in FAssignedValues);
  FAssignedValues := [];
  RefreshDefaultFont;
  RefreshDefaultHiliteFont;
  FBrush.Assign(DefaultBrush);
  Changed(FontAssigned);
end;

procedure TMenu97ItemEx.SetAlignment(Value: TAlignment);
begin
  if (Alignment <> Value) and (Index <> 0) then
  begin
    FAlignment:= Value;
    Changed(False);
  end;
end;

procedure TMenu97ItemEx.FontChanged(Sender: TObject);
begin
  Include(FAssignedValues, mvFont);
  Changed(False);
end;

procedure TMenu97ItemEx.HiliteFontChanged(Sender: TObject);
begin
  Include(FAssignedValues, mvHiliteFont);
  Changed(False);
end;

function TMenu97ItemEx.IsBitmapStored: Boolean;
begin
  Result:= (mvBitmap in FAssignedValues);
end;

function TMenu97ItemEx.IsFontStored: Boolean;
begin
  Result := (mvFont in FAssignedValues);
end;

function TMenu97ItemEx.IsHiliteFontStored: Boolean;
begin
  Result := (mvHiliteFont in FAssignedValues);
end;

function TMenu97ItemEx.IsBrushStored: Boolean;
begin
  Result := (mvBrush in FAssignedValues);
end;

procedure TMenu97ItemEx.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
  Include(FAssignedValues, mvFont);
  Changed(False);
end;

procedure TMenu97ItemEx.SetHiliteFont(Value: TFont);
begin
  FHiliteFont.Assign(Value);
  Include(FAssignedValues, mvHiliteFont);
  Changed(False);
end;

function TMenu97ItemEx.GetFont: TFont;
var
  Save: TNotifyEvent;
begin
  if not (mvFont in FAssignedValues) and (FFont.Handle <> DefaultFont.Handle) then
  begin
    Save := FFont.OnChange;
    FFont.OnChange := nil;
    FFont.Assign(DefaultFont);
    FFont.OnChange := Save;
  end;
  Result := FFont;
end;

function TMenu97ItemEx.GetHiliteFont: TFont;
var
  Save: TNotifyEvent;
begin
  if not (mvHiliteFont in FAssignedValues) and (FHiliteFont.Handle <> DefaultHiLiteFont.Handle) then
  begin
    Save := FHiliteFont.OnChange;
    FHiliteFont.OnChange := nil;
    FHiliteFont.Assign(DefaultHiLiteFont);
    FHiliteFont.OnChange := Save;
  end;
  Result := FHiliteFont;
end;

procedure TMenu97ItemEx.SetMenuItem(AMenuItem : TMenuItem);
begin
  if ( Assigned(Menu97) and Assigned(Menu97.InternalMenu) ) then begin
    if ( Assigned(FMenuItem) ) then begin
      ModifyMenu( Menu97.InternalMenu.Handle, FMenuItem.Command,
                  MF_STRING or MF_BYCOMMAND, FMenuItem.Command, PChar(FMenuItem.Caption));
      FMenuName:= '';
    end;
    FMenuItem:= AMenuItem;
    MenuName:= '';
    if ( Assigned(FMenuItem) ) then begin
      MenuName:= FMenuItem.Name;
      ModifyMenu( Menu97.InternalMenu.Handle, AMenuItem.Command,
                 MF_OWNERDRAW or MF_BYCOMMAND, AMenuItem.Command, Pointer(Menu97));
    end;
  end;
  Changed(False);
end;

procedure TMenu97ItemEx.SetBitmap(Value: TBitmap);
begin
  if ( Value = nil ) then
    Exclude(FAssignedValues, mvBitmap)
  else
    Include(FAssignedValues, mvBitmap);
  FBitmap.Assign(Value);
  Changed(False);
end;

function TMenu97ItemEx.GetMenu97: TCustomMenus97;
begin
  if Assigned(Collection) and (Collection is TMenu97ItemsEx) then
    Result := TMenu97ItemsEx(Collection).Menu97
  else
    Result := nil;
end;

function TMenu97ItemEx.GetDisplayName: string;
begin
  Result := FMenuName;
  if Result = '' then Result := inherited GetDisplayName;
end;

{ TMenuItemsEx }

constructor TMenu97ItemsEx.Create(AOwner: TCustomMenus97);
begin
  inherited Create(TMenu97ItemEx);
  FMenu97 := AOwner;
end;

function TMenu97ItemsEx.GetItem(Index: Integer): TMenu97ItemEx;
begin
  Result := TMenu97ItemEx(inherited GetItem(Index));
end;

procedure TMenu97ItemsEx.SetItem(Index: Integer; Value: TMenu97ItemEx);
begin
  inherited SetItem(Index, Value);
end;

function TMenu97ItemsEx.GetItemEx(Index : TMenuItem): TMenu97ItemEx;
Var i : Integer;
begin
  Result:= nil;
  for i:= 0 to Count - 1 do
    if ( Items[i].MenuItem = Index ) then begin
      Result:= Items[i];
      Exit;
    end;
end;

function TMenu97ItemsEx.Add: TMenu97ItemEx;
begin
  Result:= TMenu97ItemEx(inherited Add);
end;

function TMenu97ItemsEx.GetHiliteFont (index : TMenuItem) : TFont;
Var M97Item: TMenu97ItemEx;
begin
  Result:= Menu97.HiliteFont;
  M97Item:= ItemByItem[Index];
  if Assigned(M97Item) then
    Result:= M97Item.HiliteFont;
end;

function TMenu97ItemsEx.GetFont (index : TMenuItem) : TFont;
Var M97Item: TMenu97ItemEx;
begin
  Result:= Menu97.Font;
  M97Item:= ItemByItem[Index];
  if Assigned(M97Item) then
    Result:= M97Item.Font;
end;

function TMenu97ItemsEx.GetBrush (index : TMenuItem) : TBrush;
Var M97Item: TMenu97ItemEx;
begin
  Result:= Menu97.Brush;
  M97Item:= ItemByItem[Index];
  if Assigned(M97Item) then
    Result:= M97Item.Brush;
end;

function TMenu97ItemsEx.GetAlignment (index : TMenuItem) : TAlignment;
Var M97Item: TMenu97ItemEx;
begin
  Result:= taLeftJustify;
  M97Item:= ItemByItem[Index];
  if Assigned(M97Item) then
    Result:= M97Item.Alignment;
end;

function TMenu97ItemsEx.GetBitmaps (index : TMenuItem) : TBitmap;
Var M97Item: TMenu97ItemEx;
begin
  Result:= nil;
  M97Item:= ItemByItem[Index];
  if Assigned(M97Item) then
    Result:= M97Item.Bitmap;
end;

function TMenu97ItemsEx.GetImageIndex(Index: TMenuItem): Integer;
Var M97Item: TMenu97ItemEx;
begin
  Result:= -1;
  M97Item:= ItemByItem[Index];
  if Assigned(M97Item) then
    Result:= M97Item.ImageIndex;
end;

function TMenu97ItemsEx.GetOwner: TPersistent;
begin
  Result:= FMenu97;
end;

/////////////

constructor TCustomMenus97.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

     // Si le menu est un MainMenu les messages arrivent  la fentre parente
     // donc on intercepte le gestionnaire d'vennements de la fentre parente

     // if We have a main menu we need to intercept TForm messages
     if ( AOwner is TForm ) then begin
       FNewTFormWndProcInstance := MakeObjectInstance(NewTFormWndProc);
       // Keep the old DefaultProc
       FOldTFormWndProc := Pointer(GetWindowLong((Owner as TForm).Handle, GWL_WNDPROC));
       // Set the new Proc
       SetWindowLong((Owner as TForm).Handle, GWL_WNDPROC, Longint(FNewTFormWndProcInstance));
       if ( ( AOwner as TForm ).FormStyle = fsMDIForm ) then begin
         FNewTFormMDIClientWndProcInstance := MakeObjectInstance(NewTFormMDIClientWndProc);
         // Keep the old DefaultProc
         FOldTFormMDICLientWndProc := Pointer(GetWindowLong((Owner as TForm).ClientHandle, GWL_WNDPROC));
         // Set the new Proc
         SetWindowLong((Owner as TForm).ClientHandle, GWL_WNDPROC, Longint(FNewTFormMDIClientWndProcInstance));
       end;
     end;
   //end;
   FImages:= nil;
   FImageChangeLink := TChangeLink.Create;
   FMenu:= nil;
   FCanvas:= TCanvas.Create;
   FFont:= TFont.Create;
   FHiliteFont:= TFont.create;
   FBannerFont:= TFont.create;
   FBannerFont.Name:= 'Arial';
   FBannerFont.Size:= 12;
   FBannerWidth:= 20;

   // Par dfaut les deux fontes sont identiques
   FHiliteFont.Assign(FFont);
   FHiliteFont.Color:= clHighlightText;
   // Couleur de slection windows par dfaut
   // Standard Windows Color for selection
   FHiliteBar:= clHighlight;

   FBrush:= TBrush.Create;
   FBrush.Color:= clBtnFace;
   FOnDrawItem:= nil;
   FOnMeasureItem:= nil;

   FDefaultMargin:= GetSystemMetrics(SM_CXMENUCHECK);
   //CalcHeightItem;

   //Chargement de l'image pour l'option Checked & RadioItem.
   FCheckBitmap:=Tbitmap.Create;
   FRadioBitmap:=Tbitmap.Create;
   FCheckBitmap.LoadFromResourceName(HInstance,'CheckMark_V2');
   FRadioBitmap.LoadFromResourceName(HInstance,'RadioMark_V2');
   FTempMenuItem:= TMenuItem.Create(nil);
   FItemsEx:= TMenu97ItemsEx.Create(Self);
   //FItemsEx.Menu97:= Self;
end;

destructor TCustomMenus97.Destroy;

begin
  if ( Owner is TForm ) then begin
    //We have to set back the old default Proc of the form
    SetWindowLong((Owner as TForm).Handle, GWL_WNDPROC, Longint(FOldTFormWndProc));
    // and to release Our WndProc instance
    FreeObjectInstance(FNewTFormWndProcInstance);
    if ( ( Owner as TForm ).FormStyle = fsMDIForm ) then begin
      //We have to set back the old default Proc of the form
      SetWindowLong((Owner as TForm).ClientHandle, GWL_WNDPROC, Longint(FOldTFormMDIClientWndProc));
      // and to release Our WndProc instance
      FreeObjectInstance(FNewTFormMDIClientWndProcInstance);
    end;
  end;
  FImageChangeLink.Free;
  FOnDrawItem:= nil;
  FOnMeasureItem:= nil;
  FTempMenuItem.Free;
  //FItemsEx.Free;
  //FItemsEx:= nil;
  FFont.Free;
  FBannerFont.Free;
  FBrush.Free;
  FCanvas.Free;
  FMenu:= nil;
  FHiliteFont.Free;
  FCheckBitmap.free;
  FRadioBitmap.free;
  inherited Destroy;
end;

procedure TCustomMenus97.SetImages (Value: TImageList);
begin
  if ( FImages <> Value ) then begin
    if Assigned (FImages) then
      Images.UnRegisterChanges(FImageChangeLink);
    FImages := Value;
    if ( FImages <> nil ) then begin
      FDefaultMargin:= FImages.Width + 4;
      FDefaultHeight:= FImages.Height + 4;
    end;
  end;
end;

function TCustomMenus97.FindItemByCommand(Command: Integer): TMenuItem;
var C: Array[0..10] of Char;
begin
  Result:= FMenu.FindItem(Command, fkCommand);
  if ( Result = nil ) then begin
    GetMenuString(FMenu.Handle, Command, @C, 100, MF_BYCOMMAND);
    FTempMenuItem.Caption:= 'Essai' + IntToStr(Command);

    //FTempMenuItem.Parent:= FMenu;
    FTempMenuItem.Checked:= ( GetMenuState(FMenu.Handle, Command, MF_BYCOMMAND) and MF_CHECKED ) = MF_CHECKED;
    Result:= FTempMenuItem;
  end;
end;

// Calcul de la largeur maxi du menu (avec accelerateurs et sous-menus)
// Max Menu Width Calculation
function TCustomMenus97.CalcMenuWidth(Item: TMenuItem): Integer;
Var
  i: Integer;
  MaxTextWithAccel: Integer;
  MaxText: Integer;
  MaxAccel: Integer;
  TmpTextWidth: Integer;
  TmpAccelWidth: Integer;
  ParentItem: TMenuItem;

begin
  Result:= 0;
  //if ( csDesigning in ComponentState ) then Exit;
  if ( Canvas.Handle = 0 ) then Exit;
  //CalcHeightItem;
  MaxTextWithAccel:= 0;
  MaxText:= 0;
  MaxAccel:= 0;
  ParentItem:= Item.Parent;
  if (Item.Parent = FMenu.items) then
    Result:= Canvas.TextWidth(Item.Caption)
  else begin
    //ParentItem:= Item.Parent;
    if ( ParentItem = nil ) then ParentItem:= Item;
    for i:= 0 to ParentItem.Count - 1 do
      begin
        TmpTextWidth:= Canvas.TextWidth(ParentItem[i].Caption);
        TmpAccelWidth:= Canvas.TextWidth(ShortCutToText(ParentItem[i].ShortCut));
        if MaxText < TmpTextWidth then MaxText:= TmpTextWidth;
        if ( MaxTextWithAccel < TmpTextWidth ) and ( TmpAccelWidth > 0 ) then
           MaxTextWithAccel:= TmpTextWidth;
        if ( MaxAccel < TmpAccelWidth ) then  MaxAccel:= TmpAccelWidth;
      end;
    if ( ( MaxTextWithAccel + MaxAccel + FDefaultMargin ) > MaxText ) and ( MaxAccel > 0 ) then
      Result:= MaxTextWithAccel + MaxAccel + 2*FDefaultMargin
    else Result:= MaxText + 1*FDefaultMargin;
  end;
  if pos('&', Item.Caption) > 0 then Result:= Result - Canvas.TextWidth('&');
  if ( ( InternalMenu.Items[0] = ParentItem ) And ( Banner ) ) then
    Result:= Result + FBannerWidth;

end;
(* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)
{ Calculate the max Height for MenuItem. }
Procedure TCustomMenus97.CalcHeightItem;
var
   Values : Array[1..3] Of Byte;
   TestOk : Boolean;
   i : Byte;
begin
     Values[1]:=18;                               // Bitmap Height;
     Values[2]:=Font.size - Font.Height-3;        // Normal Font Height.
     Values[3]:=FHiliteFont.size - FHiliteFont.Height-3;  // Hilite Font Height.
     TestOk:=False;
     i:=1;
     While TestOk=False do
           begin
                If ( (values[i]>=Values[1]) and
                     (values[i]>=Values[2]) and
                     (values[i]>=Values[3]))  then TestOk:=True
                Else Inc(i);
           end;
     FDefaultHeight:=Values[i];
end;
(* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)
function TCustomMenus97.ProcessAccel( Message: TWMMenuChar ): Integer;

 function FindAccelerator(MenuItems: TMenuItem): Integer;
  var i: Integer;

  begin
    Result:= -1;
    if ( csDesigning in ComponentState ) or ( Not ( MenuItems.Enabled ) )  then Exit;
    for i:= 0 to MenuItems.Count - 1 do
      begin
        if ( Result = -1 ) and ( MenuItems.Enabled ) then
          begin
            if ( MenuItems.Handle <> Message.Menu ) then Result:= FindAccelerator(MenuItems[i]);
            if ( MenuItems.Handle = Message.Menu ) then
            if ( IsAccel(Word(Message.User), MenuItems[i].Caption) ) then
               Result:= MenuItems[i].MenuIndex
          end;
      end;
  end;

begin
  Result:= -1;
  if ( FMenu <> nil ) then Result:= FindAccelerator(FMenu.Items);
end;

procedure TCustomMenus97.ModifyMenuTree(MenuItems : TMenuItem; Restore: Boolean);
Var
  i: Integer;
begin
  // Parcours de l'arborescence du menu pour dfinir tous les items
  // comme OWNERDRAWN
  // All Items are defined as OWNERDRAWN or back to MF_STRING
  if ( MenuItems = nil ) then Exit;
  if ( csDesigning in ComponentState ) then Exit;
  for i:= 0 to MenuItems.Count - 1 do begin
    ModifyMenuTree(MenuItems[i], Restore);
    if ( Menuitems[i].caption <> '-' ) then begin
      if ( Restore ) then
        ModifyMenu( FMenu.Handle, MenuItems[i].Command,
                    MF_STRING or MF_BYCOMMAND, MenuItems[i].Command, PChar(MenuItems[i].Caption))
      else
        ModifyMenu( FMenu.Handle, MenuItems[i].Command,
                    MF_OWNERDRAW or MF_BYCOMMAND, MenuItems[i].Command, Pointer(Self));
    end
  end;
end;

procedure TCustomMenus97.SetInternalMenu(Value: TMenu);
begin
  if ( FMenu <> Value ) then begin
    // we need to restore Menu's original state
    FMenu:= Value;
    if not (csLoading in ComponentState) then FItemsEx.Clear;
  end;
end;

procedure TCustomMenus97.Notification;
var
   i : integer;
begin
     inherited Notification (AComponent, Operation);
     if (Operation = opRemove) and (AComponent is TMenuItem) then begin
       for i := 0 to FItemsEx.Count - 1 do
         if FItemsEx[i].MenuItem = AComponent then begin
           //FItemsEx[i].Delete;
           break;
         end;
     end;
     if (Operation = opRemove) and (AComponent = InternalMenu) then begin
       InternalMenu := nil;
       FItemsEx.Clear;
     end;
end;

procedure TCustomMenus97.FixupReferences;
var
  i : integer;
  Item : TMenuItem;
  MName : string;
begin
   if InternalMenu <> nil then begin
     for i := 0 to FItemsEx.Count - 1 do begin
       MName := FItemsEx[i].MenuName;
       Item := TMenuItem(Owner.FindComponent(MName));
       FItemsEx[i].MenuItem:= Item;
     end;
   end;
end;

procedure TCustomMenus97.Loaded;
begin
  inherited Loaded;
  //FixupReferences;
end;

(* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)

procedure TCustomMenus97.WMDrawItem(var Message: TWMDrawItem);
begin
  CNDrawItem(Message);
end;

procedure TCustomMenus97.CNDrawItem(var Message: TWMDrawItem);
var
  State     : TOwnerDrawState;
  SavedDC   : Integer;
  Item      : TMenuItem;
begin
  if ( InternalMenu = nil ) then Exit;
  with Message.DrawItemStruct^ do begin
    State := TOwnerDrawState(WordRec(LongRec(itemState).Lo).Lo);
    SavedDC:= SaveDC(hDC);
    try
      FCanvas.Handle:= hDC;
      FCanvas.Font:= FFont;
      FCanvas.Brush:= FBrush;
      if (Integer(itemID) >= 0) and (odSelected in State) then begin
        FCanvas.Brush.Color := clHighlight;
        FCanvas.Font.Color := clHighlightText;
      end;
      if Integer(itemID) >= 0 then begin
        Item:= FindItemByCommand(itemID);//FMenu.FindItem(itemID, fkCommand);
        FCanvas.Font:= FItemsEx.Font[Item];
        FCanvas.Brush:= FItemsEx.Brush[Item];
        DrawItem(Item, rcItem, State);
      end
      else
        FCanvas.FillRect(rcItem);
      if odFocused in State then
        DrawFocusRect(hDC, rcItem);
    finally
      FCanvas.Handle := 0;
      RestoreDC(hDC, SavedDC);
    end;
  end;
  Message.Result:= longint(True);
end;

procedure TCustomMenus97.DrawItem(Item: TMenuItem; Rect: TRect; State: TOwnerDrawState);

  function CalcButtonRect(ARect: TRect; BmpWidth, BmpHeight: Integer): TRect;
  begin
    InflateRect(ARect, -2, -2);
    ARect.Left:= ARect.Left - 2;
    ARect.Top := ARect.Top  - 2;
    ARect.Right:= ARect.Left + BmpWidth + 3;
    ARect.Bottom := ARect.Top + BmpHeight + 3;
    Result:= ARect;
  end;

  procedure DrawButton(Rect: TRect; Up: Boolean);
  begin
    //if ( ExItems.ItemStyle[Item] = msOffice97 ) then
    if Not ( Assigned(FItemsEx.Bitmaps[Item]) and ( Not FItemsEx.Bitmaps[Item].Empty ) ) then begin
      Rect.Right:= Rect.Right + 1;
      Canvas.FillRect(Rect);
      Rect.Right:= Rect.Right - 1;
    end;
    if (Up) then
      Frame3D(Canvas,Rect,clBtnHighlight,clBtnShadow,1)
    else
      Frame3D(Canvas,Rect,clBtnShadow,clBtnHighLight,1);
  end; {Fin.}

  procedure DrawItemText(Text: String);
  begin
    if ( Item.Default ) then
      Canvas.Font.Style:= [fsBold];
    Canvas.Brush.Style := bsClear;
    if not (Item.Enabled) then begin
      OffsetRect(Rect,1,1);
      Canvas.Font.color:= ClbtnHighlight;
    end;
    DrawText( Canvas.Handle, PChar(Text),-1, Rect,
              DT_LEFT or DT_VCENTER or DT_SINGLELINE);
    if ( Item.ShortCut <> 0 ) then begin
      Dec(Rect.Right,FDefaultMargin);
      DrawText( Canvas.Handle, PChar(ShortCutToText(Item.ShortCut)),-1,
                Rect, DT_RIGHT or DT_VCENTER or DT_SINGLELINE);
    end;
    if not (Item.Enabled) then begin
      Canvas.Font.Color:= ClbtnShadow;
      OffsetRect(Rect,-1,-1);
      DrawText( Canvas.Handle, PChar(Text),-1, Rect,
                DT_LEFT or DT_VCENTER or DT_SINGLELINE);
      if ( Item.ShortCut <> 0 ) then
        DrawText( Canvas.Handle, PChar(ShortCutToText(Item.ShortCut)),
                  -1, Rect, DT_RIGHT or DT_VCENTER or DT_SINGLELINE);
    end;
  end;

  procedure DrawMenuBarItem;
  begin
    with Canvas do begin
      //Brush.Color:=FBrush.Color;
      Brush:= FItemsEx.Brush[Item];
      Fillrect(Rect);
      if ( odSelected in State ) then
        Frame3D(Canvas,Rect,clBtnShadow,clBtnHighlight,1)
      else
        Frame3D(Canvas,Rect,Brush.color,Brush.color,1);
      Inc(Rect.Left,5);
      //Font.Assign(FFont);
      Font:= FItemsEx.Font[Item];
      DrawItemText(Item.Caption);
    end;
  end;

  procedure DrawItemBkGnd;
  Var Bmp : TBitmap;
      APen: TPen;
  begin
    Canvas.Brush:= FItemsEx.Brush[Item];
    if ( odSelected in State ) and ( Item.Enabled ) and ( Item.Caption[1] <> '-' ) then
      Canvas.Brush.Color:=FHiliteBar;
    if ( Assigned(FItemsEx.Bitmaps[Item]) and ( Not FItemsEx.Bitmaps[Item].Empty ) ) then begin
      try
        Bmp:= TBitmap.Create;
        Bmp.Width:= Rect.Right - Rect.Left;
        Bmp.Height:= Rect.Bottom - Rect.Top;
        Bmp.Canvas.StretchDraw(Bounds(0, 0, Bmp.Width,Bmp.Height), FItemsEx.Bitmaps[Item]);
        Canvas.BrushCopy( Bounds(Rect.Left,Rect.Top,Bmp.Width,Bmp.Height),Bmp,
                          Bounds(0,0,Bmp.Width,Bmp.Height), Bmp.Canvas.Pixels[0,0]);
      finally
        Bmp.Free;
      end;
    end
    else
      Canvas.Fillrect(Rect);
    if ( Item.Caption[1] = '-' ) then begin
      Item.Enabled:= False;
      try
        APen:= TPen.Create;
        APen.Style:= psSolid;
        APen.Color:= clBtnShadow;
        Canvas.Pen:= APen;
        Canvas.MoveTo ( Rect.Left+1, Rect.Top + ( Rect.Bottom - Rect.Top )  Div 2 );
        Canvas.LineTo ( Rect.Right-1, Rect.Top + ( Rect.Bottom - Rect.Top )  Div 2 );
        Canvas.Pen.Color:= clBtnHighlight;
        Canvas.MoveTo ( Rect.Left+1, Rect.Top + ( Rect.Bottom - Rect.Top )  Div 2 + 1 );
        Canvas.LineTo ( Rect.Right-1, Rect.Top + ( Rect.Bottom - Rect.Top )  Div 2 + 1 );
      finally
        APen.Free;
      end;
    end;
    Canvas.Brush:= FItemsEx.Brush[Item];
  end;

  procedure DrawItem;
  begin
    with Item, Canvas do begin
      DrawItemBkGnd;
      //if ( DrawSeparator ) then Exit;
      if ( Item.Enabled ) then begin
        if ( odSelected in State ) and ( ( Checked ) or ( FItemsEx.ImageIndex[Item] >= 0 ) ) then
          DrawButton(CalcButtonRect(Rect, FRadioBitmap.Width, FRadioBitmap.Height), True);
        if (RadioItem) then begin
          if ( Not ( odSelected in State ) ) and ( Checked ) then
            DrawButton(CalcButtonRect(Rect, FRadioBitmap.Width, FRadioBitmap.Height), Not Checked);
          if ( FItemsEx.ImageIndex[Item] >= 0 ) then begin
            Fimages.Draw(Canvas, Rect.Left+1, Rect.Top+1, FItemsEx.ImageIndex[Item]);
          end
          else
            if ( Checked ) then begin
              BrushCopy( Bounds(Rect.Left+1,Rect.Top+1,FRadioBitmap.Width,FRadioBitmap.height),FRadioBitmap,
                         Bounds(0,0,FRadioBitmap.Width,FRadioBitmap.Height),ClWhite);
            end;
        end
        else begin
          if ( Not ( odSelected in State ) ) and ( Checked ) then
            DrawButton(CalcButtonRect(Rect, FCheckBitmap.Width, FCheckBitmap.Height), False);
          if ( FItemsEx.ImageIndex[Item] >= 0 ) then
            Fimages.Draw(Canvas, Rect.Left+1, Rect.Top+1, FItemsEx.ImageIndex[Item])
          else
            if ( Checked ) then
              brushCopy( Bounds(Rect.Left+1,Rect.Top+1,FCheckBitmap.Width,FCheckBitmap.height),FcheckBitmap,
                         Bounds(0,0,FCheckBitmap.Width,FCheckBitmap.Height),ClWhite);
        end;
      end
      else begin
        if ( FItemsEx.ImageIndex[Item] >= 0 ) then
          DrawDisabledImage(Rect, FItemsEx.ImageIndex[Item]);
      end;
      if ( odSelected in State ) then
        Font:= FItemsEx.HiliteFont[Item]
        //Font.Assign(FHiliteFont)
      else
        Font:= FItemsEx.Font[Item];
        //Font.Assign(FFont);
      if ( FImages <> nil ) then
        Inc(Rect.Left,FImages.Width+7);
      if ( Item.Caption[1] = '-' ) then
        DrawItemText(Copy(Item.Caption, 2, Length(Item.Caption)))
      else
        DrawItemText(Item.Caption);
    end;
  end;

  procedure DrawMenuBkGnd;
  Var Bmp : TBitmap;
      R1, R2: TRect;
  begin
    (*if (Item.Parent = Menu.Items[0] ) then begin
      //Rect.Left:= Rect.Left + FBannerWidth + 2;
      GetMenuItemRect(Menu.WindowHandle, Menu.Items[0].Handle, 0, R1);
      GetMenuItemRect(Menu.WindowHandle, Menu.Items[0].Handle, Menu.Items[0].Count - 1, R2);
      R1.Bottom:= R2.Bottom;
      OffsetRect(R1, -R1.Left, -R1.Top);
      //R1.Right:= R1.Left + 20;
      if FBanner then
        R1.Left:= R1.Left + FBannerWidth;
      R2:= R1;
      //OffsetRect(R1, R1.Left, R1.Top);
      //Bmp:= TBitmap.Create;
      //Bmp.Width:= R1.Right - R1.Left;
      //Bmp.Height:= R1.Bottom - R1.Top;
      Canvas.Brush.Color:= FBrush.color;
      Canvas.Fillrect(Rect);
      if ( Assigned(FItemsEx.Bitmaps[Menu.Items[0]]) and ( Not FItemsEx.Bitmaps[Menu.Items[0]].Empty ) ) then begin
        try
          Bmp:= TBitmap.Create;
          Bmp.Width:= R1.Right - R1.Left;
          Bmp.Height:= R1.Bottom - R1.Top;
          Bmp.Canvas.StretchDraw(Bounds(0, 0, Bmp.Width,Bmp.Height), FItemsEx.Bitmaps[Menu.Items[0]]);
          Canvas.BrushCopy( Bounds(Rect.Left,Rect.Top,Bmp.Width,Rect.Bottom - Rect.Top),Bmp,
                            Bounds(Rect.Left,Rect.Top,Bmp.Width,Rect.Bottom - Rect.Top), Bmp.Canvas.Pixels[0,0]);
        finally
          Bmp.Free;
        end;
      end;
    end;*)
  end;

  procedure DrawBanner;
  var Bmp : TBitmap;
      TmpRect,R1, R2: TRect;
      LogRec: TLOGFONT;
      OldFont, NewFont: HFONT;
  begin
    NewFont:= 0;
    if (Item.Parent = InternalMenu.Items[0] ) then begin
      Rect.Left:= Rect.Left + FBannerWidth + 2;
      GetMenuItemRect(InternalMenu.WindowHandle, InternalMenu.Items[0].Handle, 0, R1);
      GetMenuItemRect(InternalMenu.WindowHandle, InternalMenu.Items[0].Handle, InternalMenu.Items[0].Count - 1, R2);
      R1.Bottom:= R2.Bottom;
      OffsetRect(R1, -R1.Left, -R1.Top);
      R1.Right:= R1.Left + 20;
      R2:= R1;
      OffsetRect(R1, R1.Left, R1.Top);
      try
        Bmp:= TBitmap.Create;
        Bmp.Width:= R1.Right - R1.Left;
        Bmp.Height:= R1.Bottom - R1.Top;
        Bmp.Canvas.Brush.Color:= BannerColor;
        Bmp.Canvas.FillRect(R1);
        Bmp.Canvas.Font.Color:= FBannerFont.Color;
        GetObject(BannerFont.Handle, SizeOf(LogRec), @LogRec);
        LogRec.lfEscapement := 900;
        LogRec.lfOutPrecision := OUT_TT_ONLY_PRECIS;
        NewFont:= CreateFontIndirect(LogRec);
        OldFont:= SelectObject(Bmp.Canvas.Handle,NewFont);
        if ( FBannerImage >= 0 ) then
          R1.Bottom:= R1.Bottom - 4 - Bmp.Width; // Bitmaps Width.
        SetBkMode(Bmp.Canvas.Handle, TRANSPARENT);
        case FBannerJustification of
          bjCenter : Bmp.Canvas.TextOut(R1.Left + (R1.Right - R1.Left - Bmp.Canvas.TextHeight(FBannerText)) Div 2,
                        R1.Bottom - 1 - (R1.Bottom - R1.Top - Bmp.Canvas.TextWidth(FBannerText)) Div 2, FBannerText);
          bjBottom : Bmp.Canvas.TextOut(R1.Left + (R1.Right - R1.Left - Bmp.Canvas.TextHeight(FBannerText)) Div 2,
                        R1.Bottom - 1, FBannerText);
          bjTop    : Bmp.Canvas.TextOut(R1.Left + (R1.Right - R1.Left - Bmp.Canvas.TextHeight(FBannerText)) Div 2,
                        R1.Top + Bmp.Canvas.TextWidth(FBannerText) + 1, FBannerText);
        end;
      finally
        if ( NewFont <> 0 ) then
          DeleteObject(SelectObject(Bmp.Canvas.Handle,OldFont));
        FCanvas.Draw(R2.Left, R2.Top, Bmp);
        Bmp.Free;
      end;
    end;
  end;

begin
  if Assigned(FOnDrawItem) then begin
    if ( FOnDrawItem(FMenu, Item, Rect, State) ) then
      Exit;
  end;
  if ( Banner ) then
    DrawBanner;
  DrawMenuBkGnd;
  if ((Item.Parent = FMenu.items) and (FMenu is TMainMenu)) then
    DrawMenuBarItem
  else
    DrawItem;
end; {Fin de "TCustomMenus97.DrawItem".}

procedure TCustomMenus97.WMMeasureItem(var Message: TWMMeasureItem);
begin
  CNMeasureItem(Message);
end;

procedure TCustomMenus97.CNMeasureItem(var Message: TWMMeasureItem);
Var
  Item: TMenuItem;
begin
  if ( InternalMenu = nil ) then Exit;
  try
    FCanvas.Handle:= GetDC(InternalMenu.WindowHandle);
    FCanvas.Font:= FFont;
    with Message.MeasureItemStruct^ do begin
      Item:= FindItemByCommand(itemID);//FMenu.FindItem(itemID, fkCommand);
      FCanvas.Font:= FItemsEx.Font[Item];
      if ( Item.Caption[1] = '-' ) then
        itemHeight := Canvas.TextHeight(Item.Caption) + 2
      else
        itemHeight := FDefaultHeight;
      if ( FItemWidth = 0 ) then
        itemWidth:= CalcMenuWidth(Item)
      else
        itemWidth := FItemWidth;
      MeasureItem(Item, itemHeight, itemWidth);
      Message.Result:= longint(True);
    end;
  finally
    ReleaseDC(InternalMenu.WindowHandle, FCanvas.Handle);
    FCanvas.Handle:= 0;
  end;
end;

procedure TCustomMenus97.MeasureItem(Item: TMenuItem; var Height, Width: Integer);
begin
  if Assigned(FOnMeasureItem) then
    FOnMeasureItem(FMenu, Item, Height, Width)
end;

procedure TCustomMenus97.NewTFormWndProc(Var Message: TMessage);
Var i: Integer;
begin
  // Only used to handle WM_DRAWITEM and WM_MEASUREITEM from Menus
  if ( FMenu <> nil ) then begin
    case Message.Msg of
      WM_DRAWITEM    : if ( TWMDrawItem(Message).DrawItemStruct^.CtlType = ODT_MENU ) then
                         if ( TWMDrawItem(Message).DrawItemStruct^.itemData <> 0 ) then
                           TCustomMenus97(TWMDrawItem(Message).DrawItemStruct^.itemData).Dispatch(Message);
      WM_MEASUREITEM : if ( TWMMeasureItem(Message).idCtl = 0 ) then
                         if ( TWMMeasureItem(Message).MeasureItemStruct^.itemData <> 0 ) then
                           TCustomMenus97(TWMMeasureItem(Message).MeasureItemStruct^.itemData).Dispatch(Message);
      WM_MENUCHAR    : begin
                         i:=ProcessAccel( TWMMenuChar(Message) );
                         if ( i >= 0 ) then begin
                           Message.Result:= MAKELRESULT(i, 2);
                           Exit;
                         end;
                       end;
      WM_MDIREFRESHMENU,
      WM_INITMENU,
      WM_INITMENUPOPUP  : ;
      {WM_NCHITTEST : begin
                       MessageBeep(MB_OK);
                     end;}
      CM_MENUCHANGED :
                       begin
                         if ( FMenu <> nil ) then
                           FixupReferences;
                         //  ModifyMenuTree(FMenu.Items, False);
                       end;
      {WM_MDISETMENU : begin
                        for i:= 0 to GetMenuItemCount(HMenu(Message.lParam)) - 1 do begin
                          ModifyMenu( HMenu(Message.lParam), i,
                            MF_OWNERDRAW or MF_BYPOSITION, i, Pointer(Self));
                        end;
                      end;}

    end;
  end;
  // else we call the old DefaultProc.
  Message.Result:= CallWindowProc(FOldTFormWndProc, (Owner as TForm).Handle, Message.Msg, Message.WParam, Message.LParam);
end;

procedure TCustomMenus97.NewTFormMDIClientWndProc(Var Message: TMessage);
Var i, Cmd: Integer;
    Item: TMenuItem;
begin
  // Only used to handle WM_DRAWITEM and WM_MEASUREITEM from Menus
  if ( FMenu <> nil ) then begin
    case Message.Msg of
      (*WM_DRAWITEM    : if ( TWMDrawItem(Message).DrawItemStruct^.CtlType = ODT_MENU ) then
                         if ( TWMDrawItem(Message).DrawItemStruct^.itemData <> 0 ) then
                           TCustomMenus97(TWMDrawItem(Message).DrawItemStruct^.itemData).Dispatch(Message);
      WM_MEASUREITEM : if ( TWMMeasureItem(Message).idCtl = 0 ) then
                         if ( TWMMeasureItem(Message).MeasureItemStruct^.itemData <> 0 ) then
                           TCustomMenus97(TWMMeasureItem(Message).MeasureItemStruct^.itemData).Dispatch(Message);
      WM_MENUCHAR    : begin
                         i:=ProcessAccel( TWMMenuChar(Message) );
                         if ( i >= 0 ) then begin
                           Message.Result:= MAKELRESULT(i, 2);
                           Exit;
                         end;
                       end;
      WM_MDIREFRESHMENU,
      WM_INITMENU,
      WM_INITMENUPOPUP  : ;*)
      CM_MENUCHANGED :
                      begin
                        if ( FMenu <> nil ) then
                          FixupReferences;
                        //  ModifyMenuTree(FMenu.Items, False);
                      end;
      (*WM_MDISETMENU : begin
                        Message.Result:= CallWindowProc(FOldTFormMDIClientWndProc, (Owner as TForm).ClientHandle, Message.Msg, Message.WParam, Message.LParam);
                        if ( Message.lParam = 0 ) then Exit;
                        Item:= FMenu.FindItem(HMenu(Message.lParam), fkHandle);
                        if (Message.lParam <> 0 ) then begin
                          for i:= 0 to GetMenuItemCount(HMenu(Message.lParam)) - 1 do begin
                            if ( i >= Item.Count ) then begin
                              Cmd:= GetMenuItemID(HMenu(Message.lParam), i);
                              ModifyMenu( FMenu.Handle{HMenu(Message.lParam)}, Cmd,
                                MF_OWNERDRAW or MF_BYCOMMAND, Cmd, Pointer(Self));
                            end;
                          end;
                        end;
                        Exit;
                      end;*)
    end;
  end;
  // else we call the old DefaultProc.
  Message.Result:= CallWindowProc(FOldTFormMDIClientWndProc, (Owner as TForm).ClientHandle, Message.Msg, Message.WParam, Message.LParam);
end;

procedure TCustomMenus97.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
end;

procedure TCustomMenus97.SetBannerFont(Value: TFont);
begin
  FBannerFont.Assign(Value);
end;

procedure TCustomMenus97.SetHiliteFont(Value: TFont);
begin
  FHiliteFont.Assign(Value);
end;

procedure TCustomMenus97.SetBrush(Value: TBrush);
begin
  FBrush.Assign(Value);
end;

// De larges portions du code de cette procedure
// proviennent de l'excellent TExplorerButton
// de Fabrice Deville.
// Draw a bitmap with Disabled mode.
// ---------------------------------
procedure TCustomMenus97.DrawGrayGlyph(Canvas: TCanvas; Bmp: TBitmap; x, y: Integer);
var
   MonoBmp,TmpBmp1 : TBitmap;
   Imagelist1 :Timagelist;
   NewRect : Trect;
begin
  if Bmp = nil then exit;   //No Bitmap.
  try
    ImageList1 := TImageList.CreateSize(bmp.Width, bmp.Height);
    ImageList1.DrawingStyle := dstransparent;
    ImageList1.BkColor := clnone;
    ImageList1.AddMasked(Bmp,Bmp.Canvas.Pixels[0,0]);
    TmpBmp1:=Tbitmap.create;                         //Create Tempory Bitmap.
    Imagelist1.getbitmap(0,TmpBmp1);
    NewRect := Rect(0, 0, bmp.width,Bmp.height);
    TmpBmp1.Canvas.Brush.Color := clBtnFace;
    MonoBmp := TBitmap.Create;                      //Create Monochrome Bitmap.
    MonoBmp.Monochrome := True;
    MonoBmp.Width := bmp.width;
    MonoBmp.Height := bmp.Height;
    with TmpBmp1 do begin
      // Generate the mask in MonoBmp with FTransparentColor.
      SetBkColor(Canvas.Handle,Canvas.pixels[0,0]);
      BitBlt(MonoBmp.Canvas.Handle,0,0,Width,Height,Canvas.Handle,
             0,0,SRCCOPY);
      // and White Color.
      SetBkColor(Canvas.Handle,clWhite);
      BitBlt(MonoBmp.Canvas.Handle,0,0,Width,Height,Canvas.Handle,
             0,0,SRCPAINT);
      // and clSilver.
      SetBkColor(Canvas.Handle,clSilver);
      BitBlt(MonoBmp.Canvas.Handle,0,0,Width,Height,Canvas.Handle,
             0,0,SRCPAINT);
      // Fill TmpBmp with clbtnFace color.
      Canvas.Brush.Color:=clBtnFace;
      Canvas.FillRect(NewRect);
      // Copy MonoBmp to TmpBmp with clbtnHighlight color.
      Canvas.Brush.Color:=clBtnHighlight;
      SetTextColor(Canvas.Handle,clBlack);
      SetBkColor(Canvas.Handle,clWhite);
      BitBlt(Canvas.Handle,1,1,Width-1,Height-1,MonoBmp.Canvas.Handle,
             0,0,$00B8074A);
      // Add MonoBmp to TmpBmp with clbtnShadow color.
      Canvas.Brush.Color:=clBtnShadow;
      SetTextColor(Canvas.Handle,clBlack);
      SetBkColor(Canvas.Handle,clWhite);
      BitBlt(Canvas.Handle,0,0,Width,Height,MonoBmp.Canvas.Handle,
             0,0,$00B8074A);
    end;
    // draw Bitmap for disabled state aspect.
    ImageList1.Draw(Canvas,x,y,ImageList1.AddMasked(TmpBmp1,tmpBmp1.canvas.pixels[0,0]));
    // Free ressource used.
  finally
    MonoBmp.Free;
    TmpBmp1.Free;
    Imagelist1.Free;
  end;
end;

// De larges portions du code de cette procedure
// proviennent de l'excellent TExplorerButton
// de Fabrice Deville.
// Draw a bitmap with Disabled mode.
// ---------------------------------
procedure TCustomMenus97.DrawDisabledImage(Var Rect : TRect; IndexImage : Integer);

var
  MonoBmp: TBitmap;
  Bmp    : TBitmap;

begin
  MonoBmp:= TBitmap.Create;
  Bmp    := TBitmap.Create;
  try
    Fimages.GetBitmap(IndexImage,Bmp);

    MonoBmp.Assign(Bmp);
    MonoBmp.Canvas.Brush.Color := clBlack;
    MonoBmp.Monochrome := True;

    Canvas.Brush.Color := clBtnHighlight;
    SetTextColor(Canvas.Handle, clBlack);
    SetBkColor(Canvas.Handle, clWhite);
    BitBlt(Canvas.Handle, Rect.Left+1, Rect.Top+1, Bmp.Width, Bmp.Height,
            MonoBmp.Canvas.Handle, 0, 0, $00E20746);
    Canvas.Brush.Color := clBtnShadow;
    SetTextColor(Canvas.Handle, clBlack);
    SetBkColor(Canvas.Handle, clWhite);
    BitBlt(Canvas.Handle, Rect.Left, Rect.Top, Bmp.Width, Bmp.Height,
            MonoBmp.Canvas.Handle, 0, 0, $00E20746);
  finally
    SetBkMode(Canvas.Handle, TRANSPARENT);
    MonoBmp.Free;
    Bmp.Free;
  end
end;

procedure TCustomMenus97.DrawGlyph(Canvas:TCanvas; bmp:TBitmap; x,y:Integer);
begin
  Canvas.BRush.color:=FBrush.Color;
  Canvas.brushCopy( Bounds(X,Y,Bmp.Width,Bmp.height),Bmp,
                    Bounds(0,0,Bmp.Width,Bmp.Height),Bmp.Canvas.Pixels[0,0]);
end;

procedure TCustomMenus97.SetItemsEx(Value: TMenu97ItemsEx);
begin
  FItemsEx.Assign(Value);
end;

// TCustomMainMenus97

procedure TCustomMainMenus97.SetMainMenu(Value: TMainMenu);
begin
  InternalMenu:= Value;
end;

function TCustomMainMenus97.GetMainMenu: TMainMenu;
begin
  Result:= TMainMenu(InternalMenu);
end;

// TCustomPopupMenus97

constructor TCustomPopupMenus97.Create(AOwner: TComponent);
Var TmpHandle    : THandle;
    ProcessId    : Pointer;
    TmpProcessId : Pointer;
    CName        : Array [0..50] of Char;
begin
  // Les messages OWNERDRAW - pour les menus - sont intercepts par
  // une fentre TPUtilWindow cre par delphi (PopupList)
  // moralit il faut intercepter le gestionnaire d'venements
  // de cette fentre pour pouvoir recevoir les message de dessin et autres

  // Ce hook n'es ncessaire que pour les popup menus

  // TPUtilWindow is a @#!!##@& window created by delphi (CreateWindowHandle)
  // It's defaultProc Eats All PopupMenu Messages !!
  // So We Have to Override this problem like this :
  //
  // 1st Step : Search All Windows Called 'TPUtilWindow'
  // 2d  Step : How to choose the good one ?
  //            Create a PopupMenu (Pop)
  //            Create an item (PopItem) with Hint Property = 'TPUtilWindow'
  //            Send a Message (WM_MENUSELECT) to All Found Windows (1st Step)
  //            If Application.Hint = 'TPUtilWindow' we have the good one !!
  // 3d  Step : A Little Hook on Its WndProc

  // Menu97 V 1.30 or less didn't do this (First TPUtilWindow found was the good one)

  inherited Create(AOwner);

  Inc(FTPUCount);
  if ( FTPUCount = 1 ) then begin
    try
      Pop:= TPopupMenu.Create(Self);
      Pop.Name:= 'TPUtilPopup';
      PopItem:= TMenuItem.Create(Self);
      PopItem.Name:= 'TPUtilItem';
      PopItem.Caption:= 'TPUtilWindow';
      PopItem.Hint:= 'TPUtilWindow';
      Pop.Items.Add(PopItem);
      FTPUHandle:= FindWindow('TPUtilWindow', '');
      FTPUHandle:= GetNextWindow(FTPUHandle, GW_HWNDPREV);
      GetWindowThreadProcessId(Application.Handle, @ProcessId);
      TmpHandle:= FTPUHandle;
      repeat
        FTPUHandle:= GetNextWindow(FTPUHandle, GW_HWNDNEXT);
        Windows.GetClassName(FTPUHandle, CName, 50);
        if ( strcomp( CName, 'TPUtilWindow') = 0 ) then
          GetWindowThreadProcessId(FTPUHandle, @TmpProcessId)
        else
          TmpProcessId:= nil;
        if ( ( FTPUHandle <> 0 ) and ( ProcessId = TmpProcessId ) ) then begin
          Application.Hint:= '';
          SendMessage(FTPUHandle, WM_MENUSELECT, PopItem.Command, MAKELONG(PopItem.Command, 0));
          if ( Application.Hint <> 'TPUtilWindow' ) then
            TmpProcessId:= nil
          else
            Application.Hint:= '';
        end;
      until ( ( FTPUHandle = 0 ) or ( TmpHandle = FTPUHandle ) or ( ProcessId = TmpProcessId ) );
      // Si on a trouve la fentre on installe la procdure
      // d'interception et on sauvegarde l'ancienne.
      if ( FTPUHandle <> 0 ) and ( ProcessId = TmpProcessId ) then begin
        // Create a new proc pointer
        FNewTPUtilWndProcInstance := MakeObjectInstance(NewTPUtilWndProc);
        // Keep the old DefaultProc
        FOldTPUtilWndProc := Pointer(GetWindowLong(FTPUHandle, GWL_WNDPROC));
        // Set the new Proc
        SetWindowLong(FTPUHandle, GWL_WNDPROC, Longint(FNewTPUtilWndProcInstance));
      end
      else Dec(FTPUCount);
    finally
      Pop.Free;
    end;
  end;
end;

destructor TCustomPopupMenus97.Destroy;
begin
  Dec(FTPUCount);
  if (FTPUCount = 0 ) then begin
    // We have to set back the old default Proc
    SetWindowLong(FTPUHandle, GWL_WNDPROC, Longint(FOldTPUtilWndProc));
    // and to release Our WndProc instance
    FreeObjectInstance(FNewTPUtilWndProcInstance);
  end;
  inherited Destroy;
end;

procedure TCustomPopupMenus97.SetPopupMenu(Value: TPopupMenu);
begin
  InternalMenu:= Value;
end;

function TCustomPopupMenus97.GetPopupMenu: TPopupMenu;
begin
  Result:= TPopupMenu(InternalMenu);
end;

procedure TCustomPopupMenus97.NewTPUtilWndProc(Var Message: TMessage);
Var i: Integer;
begin
  // Only used to handle WM_DRAWITEM and WM_MEASUREITEM from Menus
  case Message.Msg of
    WM_DRAWITEM    : if ( TWMDrawItem(Message).DrawItemStruct^.CtlType = ODT_MENU ) then
                       if ( TWMDrawItem(Message).DrawItemStruct^.itemData <> 0 ) then
                          TCustomMenus97(TWMDrawItem(Message).DrawItemStruct^.itemData).Dispatch(Message);
    WM_MEASUREITEM : if ( TWMMeasureItem(Message).idCtl = 0 ) then
                       if ( TWMMeasureItem(Message).MeasureItemStruct^.itemData <> 0 ) then
                          TCustomMenus97(TWMMeasureItem(Message).MeasureItemStruct^.itemData).Dispatch(Message);
    WM_MENUCHAR    : begin
                         i:=ProcessAccel( TWMMenuChar(Message) );
                         if ( i >= 0 ) then begin
                           Message.Result:= MAKELRESULT(i, 2);
                           Exit;
                         end;
                       end;
  end;
  // else we call the old DefaultProc.
  Message.Result:= CallWindowProc(FOldTPUtilWndProc, FTPUHandle, Message.Msg, Message.WParam, Message.LParam);
end;

procedure Register;
begin
  RegisterComponents('Travail', [TMainMenus97_V2]);
  RegisterComponents('Travail', [TPopupMenus97_V2]);
end;

end.
